{% extends "main.html" %} 

{% block content %} 
  {{ super() }}
  <div class="ri-root">
    {% block ri_content %}{% endblock %}
</div>
{% endblock %}

{% block footer %}
  <div class="md-footer-meta md-typeset">
    <div class="md-footer-meta__inner md-grid">
      <div class="md-copyright">
        Made with

        <a href="https://squidfunk.github.io/mkdocs-material/" target="_blank" rel="noopener">
          Material for MkDocs
        </a>
        
        and the 
        
        <a href="https://github.com/joshhubert-dsp/reserve-it" target="_blank" rel="noopener">
          reserve-it
        </a>
        
        framework.
      </div>
    </div>
  </div>
{% endblock %}
